<?php

namespace Neutrino\Ion3Core\Database\Factories;
use Illuminate\Database\Eloquent\Factories\Factory;
use Neutrino\Ion3Core\Enums\IonLookupListingType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertySubType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertyType;
use Neutrino\Ion3Core\Enums\RESOLookupStandardStatus;

class ListingFactory extends Factory
{
    protected $model = 'App\Ion3\Models\Listing';

    public function definition(): array
    {
        $property_type = $this->faker->randomElement(RESOLookupPropertyType::class);
        if (in_array($property_type,
            [RESOLookupPropertyType::CommercialLease, RESOLookupPropertyType::ResidentialLease])) {
            // Rent/Lease
            $price = rand(1, 200) * 50;
            $listing_type = IonLookupListingType::Lease;
            $price_orig = $price + 200;
            $price_sold = $price + (rand(-2, 4) * 50);
        } else {
            // Sale
            $price = rand(1, 2500) * 1000;
            $listing_type = IonLookupListingType::Sale;
            $price_orig = $price + 5000;
            $price_sold = $price + (rand(-2, 4) * 1000);
        }

        $year = date('Y', $this->faker->dateTimeThisDecade()->getTimestamp());
        if ($property_type == RESOLookupPropertyType::Land) {
            $year = 0;
        }

        $address = $this->faker->streetAddress();
        $unit_pattern = '/\s(Suite|Unit|Apt\.?)?\s\d+/i';
        $unit_number = '';
        if (preg_match($unit_pattern, $address, $matches)) {
            $unit_number = trim($matches[0]);
        }

        $state = $this->faker->stateAbbr(); // state(true) in newer faker version;
        $latitude_range = $this->getStateLatitude($state);
        $longitude_range = $this->getStateLongitude($state);

        $acres = $this->faker->randomFloat(2, 0.25, 10);

        $list_date = $this->faker->dateTimeThisYear();
        //$list_date = \DateTime::createFromFormat('Y-m-d H:i:s', $list_date->format('Y-m-d H:i:s'));
        $today = new \DateTime();
        $dom_interval = $today->diff($list_date);
        $dom = $dom_interval->days;

        $modification_date = $this->faker->dateTimeThisMonth();
        $closed_date = null;
        $pending_date = null;

        $status = $this->faker->randomElement(RESOLookupStandardStatus::class);
        if ($status == RESOLookupStandardStatus::Closed) {
            $price_sold = $this->faker->randomElement([$price, $price_sold]);
            $closed_date = $this->faker->dateTimeThisMonth($modification_date);
        } else {
            $price_sold = 0;
        }
        if ($status == RESOLookupStandardStatus::Pending) {
            $pending_date = $this->faker->dateTimeThisMonth($modification_date);
        }

        return [
            'ion_source' => 'Manual',
            'ion_listing_type' => $listing_type,
            'originating_system_name' => 'Ion3',
            'list_aor' => 'Ion3',
            'listing_id' => $this->faker->regexify('[A-Z][0-9]{7}'),

            'unparsed_address' => $address,
            'unit_number' => $unit_number,
            'city' => $this->faker->city(),
            'state_or_province' => $state,
            'postal_code' => $this->faker->postcode($state),
            'county_or_parish' => ucwords($this->faker->city()), // Newer faker supports county()
            'country' => 'US',
            'mls_area_major' => $this->faker->randomElement(['', ucwords($this->faker->city())]),
            'mls_area_minor' => $this->faker->randomElement(['', ucwords($this->faker->word())]),
            'subdivision_name' => $this->faker->randomElement(['', ucwords(str_replace('.', '', $this->faker->text(32)))]),
            'building_name' => $this->faker->randomElement(['', ucwords(str_replace('.', '', $this->faker->text(32)))]),
            'high_school_district' => $this->faker->randomElement(['', ucwords(str_replace('.', '', $this->faker->text(32)))]),

            'property_type' => $property_type,
            'property_sub_type' => $this->faker->randomElement(RESOLookupPropertySubType::class),
            'standard_status' => $status,
            'list_price' => $price,
            'original_list_price' => $this->faker->randomElement([0, $price, $price_orig]),
            'close_price' => $price_sold,

            'buyer_agency_compensation' => $this->faker->randomElement(['2%', '']),
            'bedrooms_total' => $this->faker->numberBetween(0, 5),
            'bathrooms_full' => $this->faker->numberBetween(1, 5),
            'bathrooms_half' => $this->faker->numberBetween(0, 2),
            'bathrooms_one_quarter' => $this->faker->numberBetween(0, 1),
            'bathrooms_three_quarter' => $this->faker->numberBetween(0, 1),
            'living_area' => $this->faker->numberBetween(500, 8000),
            'lot_size_square_feet' => round($acres * 43560),
            'lot_size_acres' => $acres,

            'year_built' => $year,
            'parcel_number' => $this->faker->uuid(),
            'attribution_contact' => $this->faker->name() . ' at ' . $this->faker->phoneNumber(),
            'days_on_market' => $dom,

            'internet_entire_listing_display_yn' => true,
            'internet_address_display_yn' => true,
            'latitude' => $this->faker->latitude($latitude_range[0], $latitude_range[1]),
            'longitude' => $this->faker->longitude($longitude_range[0], $longitude_range[1]),

            'modification_timestamp' => $modification_date,
            'listing_contract_date' => $list_date,
            'close_date' => $closed_date,
            'pending_timestamp' => $pending_date,
            'ion_days_on_market_date' => $modification_date,
            'ion_duplicate_of' => 0,
        ];
    }

    protected function getStateLatitude($state) {
        return match ($state) {
            'AL' => [30.1444, 35.0080],
            'AK' => [51.2097, 71.3652],
            'AZ' => [31.3322, 37.0043],
            'AR' => [33.0041, 36.4996],
            'CA' => [32.5343, 42.0095],
            'CO' => [36.9924, 41.0034],
            'CT' => [40.9801, 42.0506],
            'DC' => [38.7916, 38.9955],
            'DE' => [38.4510, 39.8395],
            'FL' => [24.3963, 31.0009],
            'GA' => [30.3571, 35.0019],
            'HI' => [18.9104, 28.4021],
            'ID' => [41.9881, 49.0008],
            'IL' => [36.9701, 42.5083],
            'IN' => [37.7717, 41.7614],
            'IA' => [40.3755, 43.5012],
            'KS' => [36.9931, 40.0032],
            'KY' => [36.4971, 39.1475],
            'LA' => [28.9336, 33.0195],
            'ME' => [43.0554, 47.4597],
            'MD' => [37.9117, 39.7230],
            'MA' => [41.2370, 42.8868],
            'MI' => [41.6961, 48.3061],
            'MN' => [43.4994, 49.3845],
            'MS' => [30.1478, 34.9961],
            'MO' => [36.0001, 40.6136],
            'MT' => [44.3582, 49.0011],
            'NE' => [39.9999, 43.0017],
            'NV' => [35.0019, 42.0022],
            'NH' => [42.6960, 45.3055],
            'NJ' => [38.9285, 41.3574],
            'NM' => [31.3323, 37.0001],
            'NY' => [40.4961, 45.0159],
            'NC' => [33.8423, 36.5882],
            'ND' => [45.9352, 49.0007],
            'OH' => [38.4034, 42.3232],
            'OK' => [33.6192, 37.0023],
            'OR' => [41.9918, 46.2920],
            'PA' => [39.7198, 42.5142],
            'RI' => [41.1466, 42.0188],
            'SC' => [32.0333, 35.2155],
            'SD' => [42.4797, 45.9456],
            'TN' => [34.9829, 36.6781],
            'TX' => [25.8371, 36.5007],
            'UT' => [36.9979, 42.0014],
            'VT' => [42.7269, 45.0167],
            'VA' => [36.5408, 39.4660],
            'WA' => [45.5435, 49.0024],
            'WV' => [37.2015, 40.6388],
            'WI' => [42.4919, 47.0807],
            'WY' => [40.9948, 45.0059],
            'default' => [24.3963, 49.3845]
        };
    }

    protected function getStateLongitude($state) {
        return match ($state) {
            'AL' => [-88.4732, -84.8882],
            'AK' => [-179.1489, 129.9795],
            'AZ' => [-114.8183, -109.0452],
            'AR' => [-94.6179, -89.6448],
            'CA' => [-124.4820, -114.1308],
            'CO' => [-109.0603, -102.0416],
            'CT' => [-73.7278, -71.7869],
            'DC' => [-77.1198, -76.9094],
            'DE' => [-75.7887, -75.0489],
            'FL' => [-87.6349, -80.0310],
            'GA' => [-85.6052, -80.8400],
            'HI' => [-178.3347, -154.8066],
            'ID' => [-117.2430, -111.0437],
            'IL' => [-91.5131, -87.4948],
            'IN' => [-88.0997, -84.7846],
            'IA' => [-96.6397, -90.1401],
            'KS' => [-102.0518, -94.5884],
            'KY' => [-89.5715, -81.9645],
            'LA' => [-94.0431, -88.8170],
            'ME' => [-71.0843, -66.9251],
            'MD' => [-79.4877, -75.0489],
            'MA' => [-73.5081, -69.9284],
            'MI' => [-90.4186, -82.4135],
            'MN' => [-97.2391, -89.4919],
            'MS' => [-91.6550, -88.0979],
            'MO' => [-95.7747, -89.0988],
            'MT' => [-116.0500, -104.0391],
            'NE' => [-104.0535, -95.3083],
            'NV' => [-120.0057, -114.0396],
            'NH' => [-72.5572, -70.6108],
            'NJ' => [-75.5594, -73.8938],
            'NM' => [-109.0502, -103.0019],
            'NY' => [-79.7624, -71.8562],
            'NC' => [-84.3219, -75.4607],
            'ND' => [-104.0490, -96.5545],
            'OH' => [-84.8203, -80.5189],
            'OK' => [-103.0026, -94.4312],
            'OR' => [-124.5662, -116.4633],
            'PA' => [-80.5199, -74.6895],
            'RI' => [-71.8628, -71.1206],
            'SC' => [-83.3540, -78.5411],
            'SD' => [-104.0577, -96.4366],
            'TN' => [-90.3103, -81.6469],
            'TX' => [-106.6456, -93.5083],
            'UT' => [-114.0530, -109.0415],
            'VT' => [-73.4377, -71.4944],
            'VA' => [-83.6754, -75.2423],
            'WA' => [-124.8489, -116.9175],
            'WV' => [-82.6449, -77.7190],
            'WI' => [-92.8881, -86.2495],
            'WY' => [-111.0569, -104.0522],
            'default' => [-124.8489, -66.9251]
        };
    }
}
