<?php

namespace Neutrino\Ion3Core\Console\Commands;

use Illuminate\Bus\Batch;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Bus;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use Neutrino\Ion3Core\Enums\Ion3ConsoleCommandNamespace;
use Neutrino\Ion3Core\Helpers\Time;
use Neutrino\Ion3Core\Services\CWSClient;
use DateTime;
use DateTimeZone;
use DateInterval;
use Neutrino\Ion3Core\Models\Ion3ListingMedia;
use Neutrino\Ion3Core\Models\Ion3Listing;
use Neutrino\Ion3Core\Jobs\CWSImportMediaJob;
use Illuminate\Support\Facades\Artisan;
class CWSImportMedia extends Command
{
	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'cws:import-media
							{--o|--originatingSystemName= : RESO OriginatingSystemName}
							{--m|--listingId= : RESO ListingId}
							{--k|--listingKey= : RESO ListingKey}
							{--modificationTimestamp= : RESO MediaModificationTimestamp Range (\'YYYY-MM-DD[ |T]HH:MM:SS\' OR \'YYYY-MM-DD[ |T]HH:MM:SS-YYYY-MM-DD[ |T]HH:MM:SS\')}
							{--goback= : Based on RESO MediaModificationTimestamp, go back (N [second(s)|minute(s)|hour(s)|day(s)|week(s)|month(s)|year(s)])}
							{--orderby=}
							{--recentlyUpdatedListings : Process Media for recently updated listings (\'process_media\' field in \'listings\' table)}
							{--top= : How many records to return at a time}
							{--limit= : How many records to return in total}
							{--queueName= : Queue name to run jobs}
							{--offset=}
							{--dump}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = "Import Constellation1 Media";

	public function __construct()
	{
		parent::__construct();

		if (function_exists('set_time_limit')) {
			set_time_limit(0);
		}

		$ion3CoreLoggingConfigPath = base_path('vendor-local/neutrino/ion3-core/config/logging.php');
		$ion3CoreLoggingConfig = require $ion3CoreLoggingConfigPath;

		Config::set('logging.channels', array_merge(
			Config::get('logging.channels'),
			$ion3CoreLoggingConfig['channels']
		));
	}

	/**
	 * Execute the console command.
	 */
	public function handle()
	{
		try {

			Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->info('BEGIN: Import Media');

			$CWSClient = new CWSClient();

			$baseFilters = [
				"OriginatingSystemName eq '" . $this->option("originatingSystemName") . "'",
			];

			////////////////////////////////////////////////////////////////////////////////////////////////////
			// If we're doing most recently updated listings, then lets do it now and get the heck out of here.
			//
			if($this->option("recentlyUpdatedListings")) {

				$recentlyUpdatedListingKeys = Ion3Listing::where('ion_source', "cws")
				                                         ->where('originating_system_name', $this->option('originatingSystemName'))
				                                         ->where('process_media', 1)
				                                         ->orderBy('modification_timestamp', 'ASC')
				                                         ->pluck('listing_key')
				                                         ->toArray();

				if(count($recentlyUpdatedListingKeys)) {

					$batches = 0;

					foreach(array_chunk($recentlyUpdatedListingKeys, 35) AS $index => $listingKeys) {

						$extraFilters = [
							"ResourceRecordKey in ('" . implode("','", $listingKeys) . "')"
						];

						$params = [
							"\$filter" => implode(" and " , array_merge($baseFilters, $extraFilters)),
							"\$top" => $top = $this->option("top") ?? 100,
							"\$orderby" => $orderBy = $this->option("orderby") ?? "MediaKey asc",
							"\$ignorenulls" => "true",
							"\$select" => "",
						];

						$nextLink = null;

						do {

							$batch   = [];
							$results = $CWSClient->search(CWSClient::$MediaEntity, $params, $this->option("limit") ?? null, !is_null($nextLink) ? ['nextLink' => $nextLink, 'set' => $set ?? 0] : null);

							foreach($results['records'] ?? [] as $record) {
								$batch[] = new CWSImportMediaJob($record);
							}

							if(count($batch)) {

								$queueName = $this->option('queueName') ? $this->option('queueName') : ("import-media-cws-" . strtolower($this->option( "originatingSystemName")) . "-" . ( $batches % 5 ));

								Bus::batch( $batch )
								   ->allowFailures()
								   ->catch( function ( Batch $batch ) {
									   Log::channel( CWSClient::getLogChannel( CWSClient::$MediaEntity ) )->error( "CWS Media import had errors!" );
								   } )
								   ->then( function ( Batch $batch ) {
									   Log::channel( CWSClient::getLogChannel( CWSClient::$MediaEntity ) )->info( "CWS Media import finished successfully!" );
								   } )
								   ->name( "CWS Import Media (" . $this->option( "originatingSystemName" ) . " | {$queueName})" )
								   ->onQueue( $queueName )
								   ->dispatch();
							}

							$nextLink = $results['nextLink'] ?? null;
							$set = $results['set'] ?? null;

						} while(!is_null($nextLink));

						Ion3Listing::where('ion_source', "cws")
						           ->where('originating_system_name', $this->option('originatingSystemName'))
						           ->where('process_media', 1)
						           ->whereIn('listing_key', $listingKeys)
						           ->update([
							           'process_media' => 0
						           ]);

						$batches ++;
					}

					exit;

				} else {
					Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->info("No listings recently updated found...");
					exit;
				}
			}

			//$cwsMetaData = $CWSClient->getMetaData();
			$timeZone = $CWSClient->getTimeZone($this->option("originatingSystemName"));

			$maxModificationTimestamp = Ion3ListingMedia::where("ion_source", "cws");

			if($this->option("originatingSystemName")) {
				$maxModificationTimestamp = $maxModificationTimestamp->where("originating_system_name", $this->option("originatingSystemName"));
			}

			$maxModificationTimestamp = $maxModificationTimestamp->max("modification_timestamp");

			$maxModificationTimestamp = trim($maxModificationTimestamp ?? '') != '' ?
				(new DateTime($maxModificationTimestamp, new DateTimeZone($timeZone)))->sub(new DateInterval('PT5M'))->format('Y-m-d\TH:i:s') :
				(new DateTime('now', new DateTimeZone($timeZone)))->sub(new DateInterval('PT5M'))->format('Y-m-d\TH:i:s');

			$maxModificationTimestamp = Time::convertToUTC0($maxModificationTimestamp, $timeZone);

			$extraFilters = [];

			if($this->option("goback")) {
				preg_match("/([\d]+) (second|minute|hour|day|week|month|year)s?$/i", $this->option("goback"), $matches);
				if(count($matches) != 3) {
					Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->error("Invalid format for '--goback'. Must match following pattern: N [second(s)|minute(s)|hour(s)|day(s)|week(s)|month(s)|year(s)]");
					return 1;
				}
				$matches[2] .= !preg_match("/s$/i", $matches[2]) ? "s" : "";
				$ModificationTimestamp = Time::convertToUTC0((new DateTime('now', new DateTimeZone($timeZone)))->modify("-{$matches[1]} {$matches[2]}")->format('Y-m-d H:i:s'));
				$extraFilters[] = "MediaModificationTimestamp ge {$ModificationTimestamp}";
			}

			if($this->option("modificationTimestamp")) {
				if(preg_match("/^([\d]{4}-[\d]{2}-[\d]{2}[ T]{1}[\d]{2}:[\d]{2}:[\d]{2})-([\d]{4}-[\d]{2}-[\d]{2}[ T]{1}[\d]{2}:[\d]{2}:[\d]{2})$/", $this->option("modificationTimestamp"), $matches)) {
					$modificationTimestampStart = Time::convertToUTC0((new DateTime($matches[1], new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$modificationTimestampEnd = Time::convertToUTC0((new DateTime($matches[2], new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$extraFilters[] = "MediaModificationTimestamp ge {$modificationTimestampStart}";
					$extraFilters[] = "MediaModificationTimestamp le {$modificationTimestampEnd}";
				} else {
					$modificationTimestampStart = Time::convertToUTC0((new DateTime($this->option("modificationTimestamp"), new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$extraFilters[] = "MediaModificationTimestamp ge {$modificationTimestampStart}";
				}

			}

			if($this->option("listingId")) {
				$extraFilters[] = "ResourceRecordID in ('" . implode("','", explode(",", $this->option("listingId"))) . "')";
			}

			if($this->option("listingKey")) {
				$extraFilters[] = "ResourceRecordKey in ('" . implode("','", explode(",", $this->option("listingKey"))) . "')";
			}

			if($extraFilters) {
				$filters = array_merge($baseFilters, $extraFilters);
			} else {
				$filters = array_merge($baseFilters, [
					"MediaModificationTimestamp ge {$maxModificationTimestamp}"
				]);
			}

			$filter = implode(" and " , $filters);

			Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->info("FILTER: {$filter}");

			$params = [
				"\$filter" => $filter,
				"\$top" => $top = $this->option("top") ?? 100,
				"\$orderby" => $orderBy = $this->option("orderby") ?? "MediaKey asc",
				"\$ignorenulls" => "true",
				"\$select" => "",
			];

			$nextLink = null;
			$batches  = 0;

			do {

				$batch = [];

				$results = $CWSClient->search(CWSClient::$MediaEntity, $params, $this->option("limit") ?? null, !is_null($nextLink) ? ['nextLink' => $nextLink, 'set' => $set ?? 0] : null);

				if($this->option("dump")) {
					print_r($results);exit;
				}

				foreach($results['records'] ?? [] as $record) {
					$batch[] = new CWSImportMediaJob($record);
				}

				if(count($batch)) {

					$queueName = $this->option('queueName') ? $this->option('queueName') : ("import-media-cws-" . strtolower($this->option( "originatingSystemName")) . "-" . ( $batches % 5 ));

					Bus::batch( $batch )
					   ->allowFailures()
					   ->catch( function ( Batch $batch ) {
						   Log::channel( CWSClient::getLogChannel( CWSClient::$MediaEntity ) )->error( "CWS Media import had errors!" );
					   } )
					   ->then( function ( Batch $batch ) {
						   Log::channel( CWSClient::getLogChannel( CWSClient::$MediaEntity ) )->info( "CWS Media import finished successfully!" );
					   } )
					   ->name( "CWS Import Media (" . $this->option( "originatingSystemName" ) . " | {$queueName})" )
					   ->onQueue( $queueName )
					   ->dispatch();

					$batches ++;
				}

				$nextLink = $results['nextLink'] ?? null;
				$set = $results['set'] ?? null;

			} while(!is_null($nextLink));

			Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->info('END: Import Media. Check queue for ongoing import items.');

		} catch (\Exception $e) {
			Log::channel(CWSClient::getLogChannel(CWSClient::$MediaEntity))->info("\nERROR: " . $e->getMessage() . " (" . $e->getFile() . " at line " . $e->getLine() . ")");
		}

	}

	public static function transformRESOOptions(array $resoOptions): array
	{
		$options = [];

		foreach($resoOptions AS $k => $v) {
			switch(true) {
				case $k == "command":
				case $k == "dataSource":
				case $k == "help":
				case $k == "quiet":
				case $k == "verbose":
				case $k == "version":
				case $k == "ansi":
				case $k == "no-interaction":
				case $k == "env":
					break;
				default:
					$options["--{$k}"] = $v;
					break;
			}
		}

		return $options;
	}

}
