<?php

namespace Neutrino\Ion3Core\Console\Commands;

use Illuminate\Bus\Batch;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Bus;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use Neutrino\Ion3Core\Helpers\Time;
use Neutrino\Ion3Core\Services\CWSClient;
use DateTime;
use DateTimeZone;
use DateInterval;
use Neutrino\Ion3Core\Models\Ion3Office;
use Neutrino\Ion3Core\Jobs\CWSImportOfficeJob;
class CWSImportOffices extends Command
{
	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'cws:import-offices
							{--o|--originatingSystemName= : RESO OriginatingSystemName} 
							{--a|--officeAor= : RESO OfficeAOR}
							{--m|--officeMlsId= : RESO OfficeMlsId}
							{--modificationTimestamp= : RESO ModificationTimestamp Range (\'YYYY-MM-DD[ |T]HH:MM:SS\' OR \'YYYY-MM-DD[ |T]HH:MM:SS-YYYY-MM-DD[ |T]HH:MM:SS\')}
							{--goback= : Based on RESO ModificationTimestamp, go back (N [second(s)|minute(s)|hour(s)|day(s)|week(s)|month(s)|year(s)])}
							{--orderby=}
							{--top= : How many records to return at a time}
							{--limit= : How many records to return in total}
							{--offset=}
							{--dump}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = "Import Constellation1 Offices";

	public function __construct()
	{
		parent::__construct();

		if (function_exists('set_time_limit')) {
			set_time_limit(0);
		}

		$ion3CoreLoggingConfigPath = base_path('vendor-local/neutrino/ion3-core/config/logging.php');
		$ion3CoreLoggingConfig = require $ion3CoreLoggingConfigPath;

		Config::set('logging.channels', array_merge(
			Config::get('logging.channels'),
			$ion3CoreLoggingConfig['channels']
		));
	}

	/**
	 * Execute the console command.
	 */
	public function handle()
	{
		try {

			Log::channel(CWSClient::getLogChannel(CWSClient::$OfficeEntity))->info('BEGIN: Import Offices');

			$batch = [];

			$CWSClient = new CWSClient();

			//$cwsMetaData = $CWSClient->getMetaData();
			$timeZone = $CWSClient->getTimeZone($this->option("originatingSystemName"), $this->option("officeAor") ?? null);

			$maxModificationTimestamp = Ion3Office::where("ion_source", "cws");

			if($this->option("originatingSystemName")) {
				$maxModificationTimestamp = $maxModificationTimestamp->where("originating_system_name", $this->option("originatingSystemName"));
			}

			if($this->option("officeAor")) {
				$maxModificationTimestamp = $maxModificationTimestamp->where("office_aor", $this->option("officeAor"));
			}

			$maxModificationTimestamp = $maxModificationTimestamp->max("modification_timestamp");

			$maxModificationTimestamp = trim($maxModificationTimestamp ?? '') != '' ?
				(new DateTime($maxModificationTimestamp, new DateTimeZone($timeZone)))->sub(new DateInterval('PT15M'))->format('Y-m-d\TH:i:s') :
				(new DateTime('now', new DateTimeZone($timeZone)))->sub(new DateInterval('PT15M'))->format('Y-m-d\TH:i:s');

			$maxModificationTimestamp = Time::convertToUTC0($maxModificationTimestamp, $timeZone);

			$baseFilters = [
				"OriginatingSystemName eq '" . $this->option("originatingSystemName") . "'",
			];

			if($this->option("officeAor")) {
				$baseFilters[] = "OfficeAOR eq '" . $this->option("officeAor") . "'";
			}

			$extraFilters = [];

			if($this->option("goback")) {
				preg_match("/([\d]+) (second|minute|hour|day|week|month|year)s?$/i", $this->option("goback"), $matches);
				if(count($matches) != 3) {
					Log::channel(CWSClient::getLogChannel(CWSClient::$OfficeEntity))->error("Invalid format for '--goback'. Must match following pattern: N [second(s)|minute(s)|hour(s)|day(s)|week(s)|month(s)|year(s)]");
					return 1;
				}
				$matches[2] .= !preg_match("/s$/i", $matches[2]) ? "s" : "";
				$RawMlsModificationTimestamp = Time::convertToUTC0((new DateTime('now', new DateTimeZone($timeZone)))->modify("-{$matches[1]} {$matches[2]}")->format('Y-m-d H:i:s'));
				$extraFilters[] = "RawMlsModificationTimestamp ge {$RawMlsModificationTimestamp}";
			}

			if($this->option("modificationTimestamp")) {
				if(preg_match("/^([\d]{4}-[\d]{2}-[\d]{2}[ T]{1}[\d]{2}:[\d]{2}:[\d]{2})-([\d]{4}-[\d]{2}-[\d]{2}[ T]{1}[\d]{2}:[\d]{2}:[\d]{2})$/", $this->option("modificationTimestamp"), $matches)) {
					$modificationTimestampStart = Time::convertToUTC0((new DateTime($matches[1], new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$modificationTimestampEnd = Time::convertToUTC0((new DateTime($matches[2], new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$extraFilters[] = "RawMlsModificationTimestamp ge {$modificationTimestampStart}";
					$extraFilters[] = "RawMlsModificationTimestamp le {$modificationTimestampEnd}";
				} else {
					$modificationTimestampStart = Time::convertToUTC0((new DateTime($this->option("modificationTimestamp"), new DateTimeZone($timeZone)))->format('Y-m-d\TH:i:s'), $timeZone);
					$extraFilters[] = "RawMlsModificationTimestamp ge {$modificationTimestampStart}";
				}

			}

			if($this->option("officeMlsId")) {
				$extraFilters[] = "OfficeMlsId in ('" . implode("','", explode(",", $this->option("officeMlsId"))) . "')";
			}

			if($extraFilters) {
				$filters = array_merge($baseFilters, $extraFilters);
			} else {
				$filters = array_merge($baseFilters, [
					"RawMlsModificationTimestamp ge {$maxModificationTimestamp}"
				]);
			}

			$filter = implode(" and " , $filters);

			Log::channel(CWSClient::getLogChannel(CWSClient::$OfficeEntity))->info("FILTER: {$filter}");

			$params = [
				"\$filter" => $filter,
				"\$top" => $top = $this->option("top") ?? 100,
				"\$orderby" => $orderBy = $this->option("orderby") ?? "OfficeKey asc",
				"\$ignorenulls" => "true",
				"\$select" => "",
			];

			$nextLink = null;
			$batches  = 0;

			do {

				$batch = [];

				$results = $CWSClient->search(CWSClient::$OfficeEntity, $params, $this->option("limit") ?? null, !is_null($nextLink) ? ['nextLink' => $nextLink, 'set' => $set ?? 0] : null);

				if($this->option("dump")) {
					print_r($results);exit;
				}

				foreach($results['records'] ?? [] as $record) {
					$batch[] = new CWSImportOfficeJob($record);
				}

				if(count($batch)) {

					$queueName = "import-offices-cws-" . strtolower($this->option( "originatingSystemName") . ( $this->option( "officeAor" ) ? "-" . $this->option( "officeAor" ) : "" )) . "-" . ( $batches % 2 );

					Bus::batch( $batch )
					   ->allowFailures()
					   ->catch( function ( Batch $batch ) {
						   Log::channel( CWSClient::getLogChannel( CWSClient::$OfficeEntity ) )->error( "CWS Offices import had errors!" );
					   } )
					   ->then( function ( Batch $batch ) {
						   Log::channel( CWSClient::getLogChannel( CWSClient::$OfficeEntity ) )->info( "CWS Offices import finished successfully!" );
					   } )
					   ->name( "CWS Import Offices (" . $this->option( "originatingSystemName" ) . ( $this->option( "officeAor" ) ? ":" . $this->option( "officeAor" ) : "" ) . " | {$queueName})" )
					   ->onQueue( $queueName )
					   ->dispatch();

					$batches ++;
				}

				$nextLink = $results['nextLink'] ?? null;
				$set = $results['set'] ?? null;

			} while(!is_null($nextLink));

			Log::channel(CWSClient::getLogChannel(CWSClient::$OfficeEntity))->info('END: Import Offices. Check queue for ongoing import items.');

		} catch (\Exception $e) {
			Log::channel(CWSClient::getLogChannel(CWSClient::$OfficeEntity))->info("\nERROR: " . $e->getMessage() . " (" . $e->getFile() . " at line " . $e->getLine() . ")");
		}

	}

	public static function transformRESOOptions(array $resoOptions): array
	{
		$options = [];

		foreach($resoOptions AS $k => $v) {
			switch(true) {
				case $k == "command":
				case $k == "dataSource":
				case $k == "help":
				case $k == "quiet":
				case $k == "verbose":
				case $k == "version":
				case $k == "ansi":
				case $k == "no-interaction":
				case $k == "env":
					break;
				default:
					$options["--{$k}"] = $v;
					break;
			}
		}

		return $options;
	}

}
