<?php

namespace Neutrino\Ion3Core;

use App\Models\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Str;

class Ion3Utility
{
    public function getSources($prefixWithManual = false): Array
    {
        $sources = config('ion3.sources', []);

        if ($prefixWithManual) {
            $sources = ['Manual' => 'Manual'] + $sources;
        }

        return $sources;
    }

    /**
     * This is for logging
     *
     * @param Model $model
     * @return string
     */
    public function getReadableModelName(Model $model): string
    {
        // Get the class name without namespace
        $className = class_basename($model);

        // Remove 'Ion3' prefix if it exists
        if (Str::startsWith($className, 'Ion3')) {
            $className = Str::replaceFirst('Ion3', '', $className);
        }

        // Split the class name into words
        $words = preg_split('/(?=[A-Z])/', $className, -1, PREG_SPLIT_NO_EMPTY);

        // Join words to form the readable string
        $readableString = implode(' ', $words);

        return $readableString;
    }

    public function logActivity(
        Model $targetModel,
        array|null $attributes,
        Model|null $by = null,
        string $type = 'updated')
    {
        if (empty($by) && Auth::check()) {
            $causer = Auth::user();
        } else {
            $causer = $by;
        }

        return activity()
            ->performedOn($targetModel)
            ->causedBy($causer)
            ->withProperties(
                [
                    'attributes' => $attributes
                ]
            )
            ->event($type)
            ->log($type);
    }

    /**
     * @param String $action view, delete, restore
     * @param String $what user, listing, company, company::location
     * @param User $user Authorized user
     * @param Model $targetModel Model in question
     * @return bool
     */
    public function checkPermissions(String $action, String $what, User $user, Model $targetModel): bool
    {
        // Least to most restrictive order is important here
        // Any user access - typically admins, do whatever they want, don't tell me what to do
        if ($user->can("{$action}_any_{$what}")) {
            return true;
        }

        $targetModelType = class_basename($targetModel);
        // Office users can edit in same office
        if ($user->can("{$action}_office_{$what}")) {
            // True if target has at least one matching CompanyLocation that the auth user has
            $authLocationIDs = $user->companyLocations->pluck('id')->toArray();
            switch ($targetModelType) {
                case 'User':
                    $targetLocationIDs = $targetModel->companyLocations->pluck('id')->toArray();
                    $commonLocationIDs = array_intersect($authLocationIDs, $targetLocationIDs);
                    return !empty($commonLocationIDs);
                case 'CompanyLocation':
                    return in_array($targetModel->id, $authLocationIDs);
                case 'Company':
                    return $user->belongsToCompany($targetModel->id);
                case 'Listing':
                    return $targetModel->belongsToCompanyLocation($authLocationIDs);
                default:
                    return false;
            }
        }

        // Same-record permissions
        if ($user->can("{$action}_{$what}")) {
            switch ($targetModelType) {
                case 'User':
                    return $user->id === $targetModel->id;
                case 'CompanyLocation':
                    $authLocationIDs = $user->companyLocations->pluck('id')->toArray();
                    return in_array($targetModel->id, $authLocationIDs);
                case 'Company':
                    return $user->belongsToCompany($targetModel->id);
                case 'Listing':
                    return $targetModel->belongsToUser($user->id);
                default:
                    return false;
            }
        }

        return false;
    }

    public function normalizeUrl(?string $input): ?string
    {
        if (blank($input)) {
            return null;
        }

        // Trim leading/trailing spaces
        $input = trim($input);

        // Check if protocol is missing; prepend "https://"
        if (!Str::startsWith($input, ['http://', 'https://'])) {
            $input = 'https://' . $input;
        }

        // Validate URL
        return filter_var($input, FILTER_VALIDATE_URL) ? $input : null;
    }

    public function resizeImage(
        String $src,
        Int $width = 0,
        Int $height = 0,
        Bool $crop = true,
        Bool $double = false): ?string
    {
        switch (Config::get('ion3.image_provider')) {
            case 'cloudinary':
                $url = Config::get('ion3.cloudinary_url').'/image/fetch/';
                $attribs = [];
                if ($width > 0) {
                    $attribs[] = 'w_' . ($width * (($double == true) ? 2 : 1));
                }
                if ($height > 0) {
                    $attribs[] = 'h_' . ($height * (($double == true) ? 2 : 1));
                }
                $attribs[] = ($crop == true) ? 'c_fill' : 'c_fit';
                $url .= implode(',', $attribs) . '/' . $src;
                return $url;

            default:
                // PHP Intervention handled by /img route and ImageController
                $cropMode = ($crop === true) ? 'crop' : 'fit';
                $realWidth = $width;
                $realHeight = $height;
                if ($double === true) {
                    $realWidth = $realWidth * 2;
                    $realHeight = $realHeight * 2;
                }
                $url = route('ion3-resize-image', [
                    'width' => $realWidth,
                    'height' => $realHeight,
                    'crop_mode' => $cropMode,
                    'url' => urlencode($src),
                ]);
                return $url;
        }
        return '';
    }
}
