<?php

namespace Neutrino\Ion3Core\Jobs;

use Illuminate\Bus\Batchable;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;

use Neutrino\Ion3Core\Models\Ion3Listing;
use Neutrino\Ion3Core\Models\Ion3ListingExtraSearchFields;
use Neutrino\Ion3Core\Services\CWSClient;
use Neutrino\Ion3Core\Transformers\Ion3DataTransformer;
use Neutrino\Ion3Core\Transformers\Ion3ListingDataTransformer;
use Neutrino\Ion3Core\Transformers\Ion3AgentDataTransformer;
use Neutrino\Ion3Core\Transformers\Ion3MediaDataTransformer;
use Neutrino\Ion3Core\Transformers\Ion3OpenHouseDataTransformer;
use Neutrino\Ion3Core\Enums\Ion3DataSource;
use Neutrino\Ion3Core\Models\Ion3Agent;
use Neutrino\Ion3Core\Models\Ion3Office;

class CWSImportListingJob implements ShouldQueue
{
	use Batchable, Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

	public static $dataSourceHandle = "CWS";

	public $record;

	/**
	 * Create a new job instance.
	 */
	public function __construct($record)
	{
		$ion3CoreLoggingConfigPath = base_path('vendor-local/neutrino/ion3-core/config/logging.php');
		$ion3CoreLoggingConfig = require $ion3CoreLoggingConfigPath;

		Config::set('logging.channels', array_merge(
			Config::get('logging.channels'),
			$ion3CoreLoggingConfig['channels']
		));

		$this->record = $record;
	}

	/**
	 * Execute the job.
	 */
	public function handle(): void
	{
		Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("Importing CWS Listing (ListingId: {$this->record['ListingId']} | ListingKey: {$this->record['ListingKey']}) from {$this->record['OriginatingSystemName']}" . (isset($this->record['ListAOR']) ? ":{$this->record['ListAOR']}" : "") .  ($this->job ? " (Queue: " . $this->job->getQueue() . ")" : ""));

		$ListingDataTransformer = new Ion3ListingDataTransformer();
		$transformedListingData = $ListingDataTransformer->transformData(Ion3DataSource::CWS->getKey(), $this->record);

		$baseListingData  = $transformedListingData['base'];
		$extraListingData = $transformedListingData['extra'];

		$Listing = Ion3Listing::updateOrCreate([
			'ion_source' => $baseListingData['ion_source'],
			'originating_system_name' => $baseListingData['originating_system_name'],
			'listing_key' => $baseListingData['listing_key'],
		], $baseListingData);

		Ion3ListingExtraSearchFields::updateOrCreate([
			'listing_id' => $Listing->id
		], $extraListingData);

		/////////////////////
		// Associate Agents
		//
		$agentKeyFields = [];
		foreach(['ListAgent', 'CoListAgent', 'BuyerAgent', 'CoBuyerAgent'] AS $prefix) {
			switch(true) {
				case isset($this->record["{$prefix}Key"]):
					$agentKeyFields[] = "{$prefix}Key";
					break;
				case isset($this->record["{$prefix}MlsId"]):
					$agentKeyFields[] = "{$prefix}MlsId";
					break;
			}
		}

		$listingAgentSyncs = [];
		foreach($agentKeyFields as $agentKeyField) {

			$position        = preg_match("/^co/i", $agentKeyField) ? 2 : 1;
			$type            = preg_match("/list/i", $agentKeyField) ? "selling" : "buying";
			$ion3LookUpField = preg_match("/mlsid/i", $agentKeyField) ? "member_mls_id" : "member_key";
			$resoLookUpField = preg_match("/mlsid/i", $agentKeyField) ? "MemberMlsId" : "MemberKey";

			$Agent = Ion3Agent::where('originating_system_name', $transformedListingData['base']['originating_system_name'])
			                  ->where($ion3LookUpField, $this->record[$agentKeyField]);

			try {
				$Agent = $Agent->firstOrFail();
			} catch (\Exception $e) {

				$CWSClient = new CWSClient();

				$params = [
					"\$filter" => "OriginatingSystemName eq '{$transformedListingData['base']['originating_system_name']}' and {$resoLookUpField} in ('{$this->record[$agentKeyField]}')",
					"\$expand" => "",
					"\$top" => 100,
					"\$orderby" => "MemberKey asc",
					"\$ignorenulls" => "true",
					"\$select" => "",
				];

				$results = $CWSClient->search(CWSClient::$MemberEntity, $params);

				if(isset($results['records'])) {
					(new CWSImportAgentJob($results['records'][0]))->handle();
					$Agent = $Agent->firstOrFail();
				}

			}

			if($Agent instanceof (new Ion3Agent())) {
				$listingAgentSyncs[$Agent->id] = [
					'type' => $type,
					'position' => $position
				];
			} else {
				Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("No associated agent found in Ion3 or CWS (MemberMlsId: " . (isset($this->record['MemberMlsId']) ? $this->record['MemberMlsId'] : "N/A") . " | MemberKey: " . (isset($this->record['MemberKey']) ? $this->record['MemberKey'] : "N/A") . ") within {$this->record['OriginatingSystemName']}, skipping Agent association...");
			}
		}
		$Listing->agents()->sync($listingAgentSyncs);

		/////////////////////
		// Associate Offices
		//
		$officeKeyFields = [];
		foreach(['ListOffice', 'CoListOffice', 'BuyerOffice', 'CoBuyerOffice'] AS $prefix) {
			switch(true) {
				case isset($this->record["{$prefix}Key"]):
					$officeKeyFields[] = "{$prefix}Key";
					break;
				case isset($this->record["{$prefix}MlsId"]):
					$officeKeyFields[] = "{$prefix}MlsId";
					break;
			}
		}

		$listingOfficeSyncs = [];
		foreach($officeKeyFields as $officeKeyField) {

			$position        = preg_match("/^co/i", $officeKeyField) ? 2 : 1;
			$type            = preg_match("/list/i", $officeKeyField) ? "selling" : "buying";
			$ion3LookUpField = preg_match("/mlsid/i", $officeKeyField) ? "office_mls_id" : "office_key";
			$resoLookUpField = preg_match("/mlsid/i", $officeKeyField) ? "OfficeMlsId" : "OfficeKey";

			$Office = Ion3Office::where('originating_system_name', $transformedListingData['base']['originating_system_name'])
			                    ->where($ion3LookUpField, $this->record[$officeKeyField]);

			try {
				$Office = $Office->firstOrFail();
			} catch (\Exception $e) {

				$CWSClient = new CWSClient();

				$params = [
					"\$filter" => "OriginatingSystemName eq '{$transformedListingData['base']['originating_system_name']}' and {$resoLookUpField} in ('{$this->record[$officeKeyField]}')",
					"\$expand" => "",
					"\$top" => 100,
					"\$orderby" => "OfficeKey asc",
					"\$ignorenulls" => "true",
					"\$select" => "",
				];

				$results = $CWSClient->search(CWSClient::$OfficeEntity, $params);

				if(isset($results['records'])) {
					(new CWSImportOfficeJob($results['records'][0]))->handle();
					$Office = $Office->firstOrFail();
				}

			}

			if($Office instanceof (new Ion3Office())) {
				$listingOfficeSyncs[$Office->id] = [
					'type' => $type,
					'position' => $position
				];
			} else {
				Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("No associated office found in Ion3 or CWS (OfficeMlsId: " . (isset($this->record['OfficeMlsId']) ? $this->record['OfficeMlsId'] : "N/A") . " | OfficeKey: " . (isset($this->record['OfficeKey']) ? $this->record['OfficeKey'] : "N/A") . ") within {$this->record['OriginatingSystemName']}, skipping Office association...");
			}
		}
		$Listing->offices()->sync($listingOfficeSyncs);

	}
}
