<?php

namespace Neutrino\Ion3Core\Listeners;

use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Facades\Log;
use Neutrino\Ion3Core\Events\Ion3Updated;

class Ion3UpdateMetadata
{
    /**
     * Create the event listener.
     */
    public function __construct()
    {
        //
    }

    /**
     * Handle the event.
     */
    public function handle(Ion3Updated $event): void
    {
        // This is a reference in json to any original values that have been overridden previously
        $originalData = $event->model->origdata ?? [];
        $metadataToLog = ['old' => [], 'attributes' => []]; // This method is where we also log metadata changes for... reasons
        if (!$event->model->isDirty()) {
            // No new data? Skip it
            return;
        }

        $fieldsToReview = [];
        foreach ($event->model->getDirty() as $key => $value) {
            if ($key == 'metadata') {
                // Special metadata value - need to pluck the actual data out of here
                $newMetadata = json_decode($value);
                $origMetadata = $event->model->getOriginal($key);
                foreach ($newMetadata as $metaK => $metaV) {
                    $oldValue = null;
                    if (isset($origMetadata[$metaK])) {
                        $oldValue = $origMetadata[$metaK];
                    }
                    // If the values are different, store for review
                    if ($oldValue != $metaV) {
                        $metadataValueToCompare = $metaV;
                        if (is_object($metadataValueToCompare)) {
                            $metadataValueToCompare = (array)$metaV;
                        }
                        $fieldsToReview["metadata_{$metaK}"] = [
                            'old' => $oldValue,
                            'new' => $metadataValueToCompare,
                        ];
                        $metadataToLog['old'][$metaK] = $oldValue;
                        $metadataToLog['attributes'][$metaK] = $metadataValueToCompare;
                    }
                }
            } else {
                // Standard field
                // If the values are different, store for review
                if ($event->model->getOriginal($key) != $value) {
                    $fieldsToReview[$key] = [
                        'old' => $event->model->getOriginal($key),
                        'new' => $value
                    ];
                }
            }
        }
        Log::debug("Object Metadata Review: " . print_r($fieldsToReview, 1));
        foreach($fieldsToReview as $key => $values) {
            $myOldValue = $values['old'];
            $myNewValue = $values['new'];

            // Are we importing new data from this object's "source"?
            if ($event->model->isUpdatingSourceData())
            {
                // Yes, and does an "origdata" value exist, meaning that this field has been overridden by the user?
                if (array_key_exists($key, $originalData)) {
                    // Yes! Ok, update the "origdata" value to reflect the new imported data, but leave the actual
                    // updated value alone.
                    $originalData[$key] = $myNewValue;
                    if (substr($key, 0, 9) == 'metadata_') {
                        // Metadata fields
                        $metadataRawKey = substr($key, 9);
                        $metadata = $event->model->metadata;
                        // Set to the previous user-provided override value
                        $event->model->metadata = array_replace($metadata, [$metadataRawKey => $myOldValue]);

                        // Skip logging cuz nothing actually changed
                        unset($metadataToLog['old'][$metadataRawKey]);
                        unset($metadataToLog['attributes'][$metadataRawKey]);
                    } else {
                        // Standard fields
                        $event->model->{$key} = $myOldValue; // Set to the previous user-provided override value
                    }
                }
            } else {
                // This is a manual update. Is the new value different than what was already there?
                if ($myNewValue != $myOldValue) {
                    // This is the first time we've overridden this value, so store the original in "origdata" to indicate it has been changed from the original.
                    if (!array_key_exists($key, $originalData)) {
                        //Log::debug("Field '{$key}' changed to [{$myNewValue}], recording [{$myOldValue}] in origdata");
                        $originalData[$key] = $myOldValue;
                    }

                    // Or, maybe the user is setting this back to the original! In that case, remove it from origdata.
                    if (array_key_exists($key, $originalData) && $originalData[$key] == $myNewValue) {
                        //Log::debug("Field '{$key}' resetting to original [{$myNewValue}] value");
                        unset($originalData[$key]);
                    }
                }
            }
        }

        // Loop through metadata and remove any null values
        if (isset($event->model->metadata) && is_array($event->model->metadata)) {
            $metadataToCleanUp = $event->model->metadata;
            foreach($metadataToCleanUp as $metaK => $metaV) {
                if (empty($metaV)) {
                    unset($metadataToCleanUp[$metaK]);
                }
            }
            $event->model->metadata = $metadataToCleanUp;
        }

        // Log activity
        if (!empty($metadataToLog['attributes']) && !$event->model->isCreatingFromSourceData()) {
            activity()
                ->performedOn($event->model)
                ->withProperties($metadataToLog)
                ->event('updated')
                ->log('updated');
        }

        $event->model->origdata = $originalData;
    }
}
