<?php

namespace Neutrino\Ion3Core\Models;

use App\Ion3\Models\Office;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Neutrino\Ion3Core\Database\Factories\CompanyLocationFactory;
use Neutrino\Ion3Core\Events\Ion3Updated;
use Neutrino\Ion3Core\Models\Ion3CompanyLocationOfficeSource;
use Neutrino\Ion3Core\Traits\Ion3MetadataTrait;
use Spatie\Activitylog\LogOptions;
use Spatie\Activitylog\Traits\LogsActivity;

class Ion3CompanyLocation extends Model
{
    use HasFactory, SoftDeletes, Ion3MetadataTrait, LogsActivity;

    protected $table = 'company_locations';
    protected $companyClass = \App\Ion3\Models\Company::class;
    protected $officeClass = \App\Ion3\Models\Office::class;
    protected $userClass = \App\Models\User::class;

    protected $guarded = ['id', 'origdata', 'deleted_at', 'created_at', 'updated_at'];
    protected $casts = [
        'metadata' => 'array',
        'origdata' => 'array',
    ];
    protected $appends = [
        'location',
    ];

    protected $dispatchesEvents = [
        'updating' => Ion3Updated::class
    ];

    protected static function newFactory()
    {
        return CompanyLocationFactory::new();
    }

    public function parent(): BelongsTo
    {
        return $this->belongsTo(self::class, 'parent_id');
    }

    public function child(): HasMany
    {
        return $this->hasMany(self::class, 'parent_id');
    }

    public function company(): BelongsTo
    {
        return $this->belongsTo($this->companyClass, 'company_id');
    }

    public function offices(): BelongsToMany
    {
        return $this->belongsToMany($this->officeClass, 'offices', 'company_location_id');
    }

    public function users(): BelongsToMany
    {
        return $this->belongsToMany($this->userClass, 'company_location_user', 'company_location_id', 'user_id')
            ->withPivot('position')
            ->withTimestamps();
    }

    public function officeSources(): HasMany
    {
        return $this->hasMany(Ion3CompanyLocationOfficeSource::class, 'company_location_id');
    }

    // Typesense Data Collection name
    public function searchableAs()
    {
        return 'company_locations';
    }

    /********** START: Used by the Filament Google Maps package ************/
    /**
     * Returns the 'latitude' and 'longitude' attributes as the computed 'location' attribute,
     * as a standard Google Maps style Point array with 'lat' and 'lng' attributes.
     *
     * Requires the 'location' attribute be included in this model's $fillable array.
     *
     * @return array
     */
    public function getLocationAttribute(): array
    {
        return [
            "lat" => (float)$this->latitude,
            "lng" => (float)$this->longitude,
        ];
    }
    /**
     * Takes a Google style Point array of 'lat' and 'lng' values and assigns them to the
     * 'latitude' and 'longitude' attributes on this model.
     *
     * Requires the 'location' attribute be included in this model's $fillable array.
     *
     * @param ?array $location
     * @return void
     */
    public function setLocationAttribute(?array $location): void
    {
        if (is_array($location))
        {
            $this->attributes['latitude'] = $location['lat'];
            $this->attributes['longitude'] = $location['lng'];
            unset($this->attributes['location']);
        }
    }

    /**
     * Get the lat and lng attribute/field names used on this table
     *
     * @return string[]
     */
    public static function getLatLngAttributes(): array
    {
        return [
            'lat' => 'latitude',
            'lng' => 'longitude',
        ];
    }

    /**
     * Get the name of the computed location attribute
     *
     * @return string
     */
    public static function getComputedLocation(): string
    {
        return 'location';
    }
    /********** END: Used by the Filament Google Maps package ************/



    public function getFilamentAvatarUrl(): ?string
    {
        // If photo_url is empty, return default
        if (empty($this->photo_url)) {
            return $this->getDefaultImage();
        }

        if (str_contains($this->photo_url, 'http')) {
            return $this->photo_url;
        }

        $disk = config('filament.default_filesystem_disk', 'public');
        if (!array_key_exists($disk, config('filesystems.disks'))) {
            $disk = 'public';
        }

        if (null !== config("filesystems.disks.{$disk}.url")) {
            return rtrim(config("filesystems.disks.{$disk}.url"), '/') . '/' . ltrim($this->photo_url, '/');
        } else {
            return Storage::disk($disk)->url($this->photo_url);
        }
    }

    static public function getDefaultImage() {
        return asset('images/default/companylocation.png');
    }

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
            ->logExcept(['origdata', 'updated_at', 'metadata']) // really important
            ->logOnly(['*'])
            ->logUnguarded()
            ->logOnlyDirty()
            ->dontSubmitEmptyLogs();
    }
}
