<?php

namespace Neutrino\Ion3Core\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\SoftDeletes;
use Neutrino\Ion3Core\Database\Factories\ListingFactory;
use Neutrino\Ion3Core\Enums\IonLookupListingType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertySubType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertyType;
use Neutrino\Ion3Core\Enums\RESOLookupStandardStatus;
use PhpParser\Node\Expr\Array_;

class Ion3Listing extends Model
{
    use HasFactory;
    use SoftDeletes;

    protected $table = 'listings';
    protected $agentClass = \App\Ion3\Models\Agent::class;
    protected $officeClass = \App\Ion3\Models\Office::class;
    protected $openHouseClass = \App\Ion3\Models\OpenHouse::class;
    protected $listingExtraSearchFieldsClass = \App\Ion3\Models\ListingExtraSearchFields::class;
    protected $listingMediaClass = \App\Ion3\Models\ListingMedia::class;
    protected $listingMetaClass = \App\Ion3\Models\ListingMeta::class;
    protected $guarded = ['id', 'origdata', 'deleted_at', 'created_at', 'updated_at'];

    protected $casts = [
        'ion_listing_type' => IonLookupListingType::class,
        'property_type' => RESOLookupPropertyType::class,
        'property_sub_type' => RESOLookupPropertySubType::class,
        'standard_status' => RESOLookupStandardStatus::class,
        'lot_size_acres' => 'float',
        'internet_entire_listing_display_yn' => 'boolean',
        'internet_address_display_yn' => 'boolean',
        'latitude' => 'float',
        'longitude' => 'float',
        'modification_timestamp' => 'datetime',
        'listing_contract_date' => 'date',
        'ion_days_on_market_date' => 'date',
        'close_date' => 'date',
        'pending_timestamp' => 'datetime',
        'year_built' => 'integer',
        'metadata' => 'array',
        'origdata' => 'array',
    ];

    static function newFactory()
    {
        return ListingFactory::new();
    }

    public function extraSearchFields(): HasOne
    {
        return $this->hasOne($this->listingExtraSearchFieldsClass);
    }

    public function meta(): HasMany
    {
        return $this->hasMany($this->listingMetaClass, 'listing_id');
    }

    public function media(): HasMany
    {
        return $this->hasMany($this->listingMediaClass, 'listing_id');
    }

    public function openHouses(): HasMany
    {
        return $this->hasMany($this->openHouseClass, 'listing_id');
    }

    public function agents(): BelongsToMany
    {
        return $this->belongsToMany($this->agentClass, 'agent_listing', 'listing_id', 'agent_id')
            ->withTimestamps()
            ->withPivot('type', 'position', 'active');
    }

    public function offices(): BelongsToMany
    {
        return $this->belongsToMany($this->officeClass, 'listing_office', 'listing_id', 'office_id')
            ->withTimestamps()
            ->withPivot('type', 'position', 'active');
    }

    public function orderedAgents()
    {
        return $this->belongsToMany($this->agentClass, 'agent_listing')
            ->wherePivot('active', 1) // Only include active agents
            ->orderByPivot('type') // Order by type in the pivot table
            ->orderByPivot('position'); // Then order by position
    }

    public function belongsToUser(int|array $userIds): bool
    {
        // Ensure input is an array
        $userIds = is_array($userIds) ? $userIds : [$userIds];

        // Get the company_location_id of all related offices
        $relatedUserIds = $this->agents->pluck('user_id')->unique()->toArray();

        // Check if any of the provided companyLocationIds are in the relatedCompanyLocationIds
        return !empty(array_intersect($userIds, $relatedUserIds));
    }

    public function belongsToCompanyLocation(int|array $companyLocationIds): bool
    {
        // Ensure input is an array
        $companyLocationIds = is_array($companyLocationIds) ? $companyLocationIds : [$companyLocationIds];

        // Get the company_location_id of all related offices
        $relatedCompanyLocationIds = $this->offices->pluck('company_location_id')->unique()->toArray();

        // Check if any of the provided companyLocationIds are in the relatedCompanyLocationIds
        return !empty(array_intersect($companyLocationIds, $relatedCompanyLocationIds));
    }

    // Typesense Data Collection name
    public function searchableAs()
    {
        return 'listings';
    }

    public function getAddress(): String
    {
        return $this->unparsed_address;
    }
}
