<?php

// API Reference: https://api.constellation1.com/listings/
namespace Neutrino\Ion3Core\Services;

use GuzzleHttp\Client;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Config;
use Carbon\Carbon;
use \DOMDocument;
use \DOMXPath;
use Illuminate\Support\Facades\Log;

class CWSClient
{
	public static $PropertyEntity = "Property";
	public static $MemberEntity = "Member";
	public static $OfficeEntity = "Office";
	public static $MediaEntity = "Media";
	public static $OpenHouseEntity = "OpenHouse";
	public static $DomEntity = "Dom";
	public static $PropertyUnitTypesEntity = "PropertyUnitTypes";
	public static $PropertyRoomsEntity = "PropertyRooms";
	public static $RawMlsPropertyEntity = "RawMlsProperty";

	public function search($entityType, $params, $limit = null, $extra = null): array
	{
		$data = [];
		$set  = !is_null($extra) && isset($extra['set']) ? $extra['set'] : 0;

		//Log::channel(self::getLogChannel($entityType))->info(print_r($params,1));

		$client = new Client();
		$url = !is_null($extra) && isset($extra['nextLink']) ? $extra['nextLink'] : Config::get('services.constellation1.search_base_url') . $entityType . "?" . http_build_query($params);

		do {

			$apiTries = 0;

			do {

				$success = true; // Let's assume the API call success - if it doesn't we'll handle it down below

				//$url = "https://listings.constellation1apis.com/OData/property?\$apply=groupby((OriginatingSystemName))&\$top=0";
				//Log::channel(self::getLogChannel($entityType))->info($url);

				$response = $client->get($url, [
					'headers' => [
						"Authorization" => "Bearer " . (new CWSTokenService())->getAccessToken(),
						"Host" => parse_url(Config::get('services.constellation1.search_base_url'), PHP_URL_HOST),
					],
				]);

				$responseBody = json_decode($response->getBody()->getContents(), true);

				//dd($responseBody);

				$apiTries++;

				if(isset($responseBody['message']) && $responseBody['message'] == "Endpoint request timed out") {
					$success = false;
					Log::channel(self::getLogChannel($entityType))->info("ERROR: {$responseBody['message']}...");
					if($apiTries < 5) {
						Log::channel(self::getLogChannel($entityType))->info(" - Tried {$apiTries} time" . ($apiTries > 1 ? "s" : "") . ", trying again...");
					} else {
						Log::channel(self::getLogChannel($entityType))->info(" - Already tried {$apiTries} times, giving up...");
					}
				}

			} while (!$success && $apiTries < 5);

			$data['totalRecordsFound'] = $responseBody["@odata.totalCount"];

			$start = ($set * $params["\$top"] ?? 0) + 1;
			$end = $start + ($responseBody["@odata.count"] - 1);

			if(!is_null($limit)) {
				$end = $limit < $end ? $limit : $end;
			}

			Log::channel(self::getLogChannel($entityType))->info("Fetched Records {$start} - {$end} of {$responseBody["@odata.totalCount"]}" . (!is_null($limit) ? " (Only fetching {$limit})" : "") . " (Memory Usage: " . $this->formatBytes(memory_get_usage()) . ")");

			foreach($responseBody['value'] as $record) {

				$data['records'][] = $record;
				$data['totalRecordsReturned'] = count($data['records']);

				if(!is_null($limit)) {
					if($data['totalRecordsReturned'] >= $limit) {
						break 2;
					}
				}
			}

			$url = $responseBody["@odata.nextLink"] ?? null;

			gc_collect_cycles();

			$set++;

			////////////////////////////////////////////////////
			// Let's only process 10,000 records max at a time
			//
			if(!is_null($url) && $data['totalRecordsReturned'] >= 10000) {
				Log::channel(self::getLogChannel($entityType))->info(" - Payload size exceeded, returning {$data['totalRecordsReturned']} records and letting another process handle the rest...");
				$data['nextLink'] = $url;
				$data['set'] = $set;
				return $data;
			}

		} while (!is_null($url));

		return $data;
	}

	function getTimezone($originatingSystemName, $listAor = null): string
	{
		$timeZones = Config::get('services.constellation1.timezones');

		if(is_null($listAor)) {
			if(is_array($timeZones[$originatingSystemName])) {
				return array_values($timeZones[$originatingSystemName])[0];
			} else {
				return $timeZones[$originatingSystemName];	
			}
		} else {
			if(isset($timeZones[$originatingSystemName][$listAor])) {
				return $timeZones[$originatingSystemName][$listAor];
			} else {
				if(is_array($timeZones[$originatingSystemName])) {
					return array_values($timeZones[$originatingSystemName])[0];
				} else {
					return $timeZones[$originatingSystemName];
				}
			}
		}
	}

	function getMetaData(): array
	{
		$metaData = [];

		if (Cache::has("cws_entity_meta_xml")) {
			$metaDataXML = Cache::get("cws_entity_meta_xml");
		} else {
			$client = new Client();
			$response = $client->get(Config::get('services.constellation1.search_base_url') . "\$metadata", [
				'headers' => [
					"Authorization" => "Bearer " . (new CWSTokenService())->getAccessToken(),
					"Host" => parse_url(Config::get('services.constellation1.search_base_url'), PHP_URL_HOST),
				]
			]);
			$metaDataXML = $response->getBody()->getContents();
		}

		Cache::put("cws_entity_meta_xml", $metaDataXML, now()->add(\Carbon\Unit::Hour, 6));

		$DOM = new DOMDocument;
		$DOM->loadXML($metaDataXML);

		$xpath = new DOMXPath($DOM);
		$xpath->registerNamespace('edmx', 'http://docs.oasis-open.org/odata/ns/edmx');
		$xpath->registerNamespace('edm', 'http://docs.oasis-open.org/odata/ns/edm');

		$metaData['Enums'] = $this->getEnumMetaData($xpath);
		$metaData['Entities'] = $this->getEntityMetaData($xpath);

		return $metaData;
	}

	function getEntityMetaData($xpath): array
	{
		$entityMetaData = [];

		foreach ($xpath->query("//edm:EntityType") as $entityType) {
			$key = $xpath->query('./edm:Key', $entityType)[0];
			$ref = $xpath->query('./edm:PropertyRef', $key)[0];
			foreach ($xpath->query('./edm:Property', $entityType) as $property) {
				$entityMetaData[$entityType->getAttribute('Name')][$property->getAttribute('Name')] = [
					"name" => $property->getAttribute('Name'),
					"data_type" => $property->getAttribute('Type'),
					"is_key" => ($property->getAttribute('Name') == $ref->getAttribute('Name'))
				];
			}
		}

		return $entityMetaData;
	}

	function getEnumMetaData($xpath): array
	{
		$enumMetaData = [];

		foreach ($xpath->query("//edm:EnumType") as $enumType) {
			$members = $xpath->query('./edm:Member', $enumType);
			foreach ($members as $member) {
				$annotations = $xpath->query('./edm:Annotation', $member);
				$enumMetaData[$enumType->getAttribute('Name')][$member->getAttribute('Name')] = [
					"internalName" => $enumType->getAttribute('Name'),
					"value" => $member->getAttribute('Name'),
					"standardName" => $annotations->length ? $annotations[0]->getAttribute('String') : $member->getAttribute('Name'),
				];
			}
		}

		return $enumMetaData;
	}

	public static function getLogChannel($entityType) {
		switch($entityType) {
			case self::$PropertyEntity:
				return "cwslistingimports";
			case self::$MemberEntity:
				return "cwsagentimports";
			case self::$OfficeEntity:
				return "cwsofficeimports";
			case self::$OpenHouseEntity:
				return "cwsopenhouseimports";
			case self::$MediaEntity:
				return "cwsmediaimports";
		}
	}

	function formatBytes($bytes, $precision = 2) {
		$units = ['B', 'KB', 'MB', 'GB', 'TB'];

		$bytes = max($bytes, 0);
		$pow = floor(($bytes ? log($bytes) : 0) / log(1024));
		$pow = min($pow, count($units) - 1);

		// Calculate the value in the appropriate unit
		$bytes /= (1 << (10 * $pow));

		return round($bytes, $precision) . ' ' . $units[$pow];
	}
}