<?php

namespace Neutrino\Ion3Core\Transformers;

use DateTime;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use Neutrino\Ion3Core\Enums\Ion3DataSource;
use Neutrino\Ion3Core\Enums\RESOLookupStandardStatus;
use Neutrino\Ion3Core\Models\Ion3Listing;
use Neutrino\Ion3Core\Models\Ion3ListingMedia;
use Neutrino\Ion3Core\Services\CWSClient;
use Neutrino\Ion3Core\Transformers\Ion3ListingDataTransformer;
use App\Jobs\BHSImportListingJob;
use Neutrino\Ion3Core\Helpers\Time;

class Ion3ListingDataFromCWSTransformer extends Ion3ListingDataTransformer
{
	public static function transform(array $data): array
	{
		Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("Transforming CWS Listing Data (ListingId: {$data['ListingId']} | ListingKey: {$data['ListingKey']}) from {$data['OriginatingSystemName']}" . (isset($data['ListAOR']) ? ":{$data['ListAOR']}" : ""));

		$transformedData = [
			'base' => [
				'ion_source'    => Ion3DataSource::CWS->getKey(),
				'process_media' => 0, // Let's assume, for now, that we don't have to process the media for this listing
			],
			'extra' => [],
		];

		foreach($data as $key => $value) {

			switch($key) {
				case "Country":
					$transformedData['base']['country'] = $value;
					break;
				case "OriginatingSystemName":
					$transformedData['base']['originating_system_name'] = $value;
					break;
				case "ListAOR":
					$transformedData['base']['list_aor'] = $value;
					break;
				case "ListingKey":
					$transformedData['base']['listing_key'] = $value;
					break;
				case "ListingId":
					$transformedData['base']['listing_id'] = $value;
					break;
				case "PropertyType":
					$transformedData['base']['property_type'] = $value;
					switch($value) {
						case "CommercialLease":
						case "ResidentialLease":
							$transformedData['base']['ion_listing_type'] = "Lease";
							break;
						default:
							if(isset($data['LeaseConsideredYN']) && $data['LeaseConsideredYN']) {
								$transformedData['base']['ion_listing_type'] = "Sale or Lease";
							} else {
								$transformedData['base']['ion_listing_type'] = "Sale";
							}
							break;
					}
					break;
				case "UnparsedAddress":
					$transformedData['base']['unparsed_address'] = $value ?? '';
					break;
				case "UnitNumber":
					$transformedData['base']['unit_number'] = $value ?? '';
					break;
				case "City":
					$transformedData['base']['city'] = $value ?? '';
					break;
				case "StateOrProvince":
					$transformedData['base']['state_or_province'] = $value ?? '';
					break;
				case "PostalCode":
					$transformedData['base']['postal_code'] = $value ?? '';
					break;
				case "CountyOrParish":
					$transformedData['base']['county_or_parish'] = $value ?? '';
					break;
				case "CityRegion":
					$transformedData['base']['area'] = $value ?? '';
					break;
				case "PropertySubType":
					$transformedData['base']['property_sub_type'] = $value ?? '';
					break;
				case "StandardStatus":
					switch(true) {
						case !in_array($value, RESOLookupStandardStatus::asArray()):
							switch($value) {
								case "Off Market":
									$transformedData['base']['standard_status'] = RESOLookupStandardStatus::Delete;
									break;
								default:
									$transformedData['base']['standard_status'] = RESOLookupStandardStatus::Active;
									break;
							}
							break;
						default:
							$transformedData['base']['standard_status'] = $value ?? "Active";
							break;
					}
					break;
				case "OriginalListPrice":
					$transformedData['base']['original_list_price'] = $value ?? 0;
					break;
				case "ListPrice":
					$transformedData['base']['list_price'] = $value ?? 0;
					break;
				case "ClosePrice":
					$transformedData['base']['close_price'] = $value ?? 0;
					break;
				case "BedroomsTotal":
					$transformedData['base']['bedrooms_total'] = $value;
					break;
				case "BathroomsTotalInteger":
					$transformedData['base']['bathrooms_full'] = $value;
					break;
				case "BathroomsPartial":
					$transformedData['base']['bathrooms_half'] = $value;
					break;
				case "LivingArea":
					$transformedData['base']['living_area'] = $value;
					break;
				case "LotSizeSquareFeet":
					$transformedData['base']['lot_size_square_feet'] = $value;
					break;
				case "LotSizeAcres":
					$transformedData['base']['lotSizeAcres'] = $value;
					break;
				case "YearBuilt":
					$transformedData['base']['year_built'] = $value ?? 0;
					break;
				case "DaysOnMarket":
					$transformedData['base']['days_on_market'] = $value;
					break;
				case "InternetEntireListingDisplayYN":
					$transformedData['base']['internet_entire_listing_display_yn'] = $value;
					break;
				case "InternetAddressDisplayYN":
					$transformedData['base']['internet_address_display_yn'] = $value;
					break;
				case "Latitude":
					$transformedData['base']['latitude'] = $value;
					break;
				case "Longitude":
					$transformedData['base']['longitude'] = $value;
					break;
				case "RawMlsModificationTimestamp":
					$transformedData['base']['modification_timestamp'] = Time::convertFromUTC0($value, (new CWSClient())->getTimezone($data['OriginatingSystemName'], isset($data['ListAOR']) ? $data['ListAOR'] : null)) ?  : null;
					break;
				case "RawMlsPhotosChangeTimestamp":
					$transformedData['base']['photo_change_timestamp'] = Time::convertFromUTC0($value, (new CWSClient())->getTimezone($data['OriginatingSystemName'], isset($data['ListAOR']) ? $data['ListAOR'] : null)) ?  : null;
					break;
				case "PhotosCount":
				case "RawMlsPhotosCount":
					$transformedData['base']['photos_count'] = $transformedData['base']['photos_count'] ?? ($value ?? 0);
					break;
				case "ListingContractDate":
					$transformedData['base']['listing_contract_date'] = $value ?? null;
					break;
				case "CloseDate":
					$transformedData['base']['close_date'] = $value ?? null;
					break;
				case "MLSAreaMajor":
					$transformedData['base']['mls_area_major'] = $value ?? '';
					break;
				case "MLSAreaMinor":
					$transformedData['base']['mls_area_minor'] = $value ?? '';
					break;
				case "SubdivisionName":
					$transformedData['base']['subdivision_name'] = $value ?? '';
					break;
				case "BuildingName":
					$transformedData['base']['building_name'] = $value ?? '';
					break;
				case "HighSchoolDistrict":
					$transformedData['base']['high_school_district'] = $value ?? '';
					break;
				case "HighSchool":
					$transformedData['base']['high_school'] = $value ?? '';
					break;
				case "ElementarySchoolDistrict":
					$transformedData['base']['elementary_school_district'] = $value ?? '';
					break;
				case "ElementarySchool":
					$transformedData['base']['elementary_school'] = $value ?? '';
					break;
				case "MiddleOrJuniorSchoolDistrict":
					$transformedData['base']['middle_or_junior_school_district'] = $value ?? '';
					break;
				case "MiddleOrJuniorSchool":
					$transformedData['base']['middle_or_junior_school'] = $value ?? '';
					break;

				////////////////////////
				// Extra Search Fields
				//
				case "AssociationFee":
					$transformedData['extra']['association_fee'] = $value;
					break;
				case "AssociationYN":
					$transformedData['extra']['association_yn'] = $value;
					break;
				case "ParcelNumber":
					$transformedData['extra']['parcel_number'] = $value;
					break;
				case "PublicRemarks":
					$transformedData['extra']['remarks'] = $value;
					break;
				case "Appliances":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['appliances'] = implode(",", $value);
					break;
				case "ArchitecturalStyle":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['architectural_style'] = implode(",", $value);
					break;
				case "Basement":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['basement'] = implode(",", $value);
					break;
				case "BasementYN":
					$transformedData['extra']['basement_yn'] = $value;
					break;
				case "CommonWalls":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['common_walls'] = implode(",", $value);
					break;
				case "ConstructionMaterials":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['construction_materials'] = implode(",", $value);
					break;
				case "Cooling":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['cooling'] = implode(",", $value);
					break;
				case "Fencing":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['fencing'] = implode(",", $value);
					break;
				case "Heating":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['heating'] = implode(",", $value);
					break;
				case "InteriorFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['interior_features'] = implode(",", $value);
					break;
				case "Levels":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['levels'] = implode(",", $value);
					break;
				case "LotFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['lot_features'] = implode(",", $value);
					break;
				case "ParkingFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['parking_features'] = implode(",", $value);
					break;
				case "PatioAndPorchFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['patio_and_porch_features'] = implode(",", $value);
					break;
				case "PoolFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['pool_features'] = implode(",", $value);
					break;
				case "RoadResponsibility":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['road_responsibility'] = implode(",", $value);
					break;
				case "Sewer":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['sewer'] = implode(",", $value);
					break;
				case "ElectricOnPropertyYN":
					$transformedData['extra']['electric_on_property_yn'] = $value;
					break;
				case "Utilities":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['utilities'] = implode(",", $value);
					break;
				case "SyndicateTo":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['syndicate_to'] = implode(",", $value);
					break;
				case "WaterSource":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['water_source'] = implode(",", $value);
					break;
				case "PoolPrivateYN":
					$transformedData['extra']['pool_private_yn'] = $value;
					break;
				case "PropertyAttachedYN":
					$transformedData['extra']['property_attached_yn'] = $value;
					break;
				case "SpaYN":
					$transformedData['extra']['spa_yn'] = $value;
					break;
				case "TaxAnnualAmount":
					$transformedData['extra']['tax_annual_amount'] = $value;
					break;
				case "FireplaceFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['fireplace_features'] = implode(",", $value);
					break;
				case "FireplacesTotal":
					$transformedData['extra']['fireplaces_total'] = $value;
					break;
				case "FireplaceYN":
					$transformedData['extra']['fireplace_yn'] = $value;
					break;
				case "LeaseTerm":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['lease_term'] = implode(",", $value);
					break;
				case "PetsAllowed":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['pets_allowed'] = implode(",", $value);
					break;
				case "SeniorCommunityYN":
					$transformedData['extra']['senior_community_yn'] = $value;
					break;
				case "SpaFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['spa_features'] = implode(",", $value);
					break;
				case "Topography":
					$transformedData['extra']['topography'] = $value;
					break;
				case "View":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['view'] = implode(",", $value);
					break;
				case "ViewYN":
					$transformedData['extra']['view_yn'] = $value;
					break;
				case "WaterBodyName":
					$transformedData['extra']['water_body_name'] = $value;
					break;
				case "WaterfrontFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['waterfront_features'] = implode(",", $value);
					break;
				case "WaterfrontYN":
					$transformedData['extra']['waterfront_yn'] = $value;
					break;
				case "OtherEquipment":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['other_equipment'] = implode(",", $value);
					break;
				case "SecurityFeatures":
					$value = is_array($value) ? $value : [ $value ];
					$transformedData['extra']['security_features'] = implode(",", $value);
					break;
			}

		}

		switch(true) {
			case isset($transformedData['base']['days_on_market']):
				$transformedData['base']['ion_days_on_market_date'] = (new \DateTime())->format('Y-m-d');
		}

		////////////////////////////////////////////////////////
		// Multiple checks to see if we need to process media
		//
		try {

			//////////////////////////////////////////////////////////////////////////////////////////////////////////
			// If we don't have this listing in our system yet, then we should process the media associated with it
			//
			$existingListing = Ion3Listing::where("ion_source", Ion3DataSource::CWS->getKey())
			                              ->where("originating_system_name", $transformedData['base']['originating_system_name'])
			                              ->where("listing_key", $transformedData['base']['listing_key'])
			                              ->firstOrFail();

			/////////////////////////////////////////////////////////////////////////////////////////////////////
			// Is the incoming 'photo_change_timestamp' different than our existing 'photo_change_timestamp'?
			// If so, let's process the media!
			//
			if((new DateTime($transformedData['base']['photo_change_timestamp'])) > (new DateTime($existingListing->photo_change_timestamp))) {
				throw new \Exception("Existing photo_change_timestamp ({$existingListing->photo_change_timestamp}) LESS THAN incoming photo_change_timestamp ({$transformedData['base']['photo_change_timestamp']})");
			}

			if(isset($transformedData['base']['photos_count']) && $transformedData['base']['photos_count'] > 0) {

				/////////////////////////////////////////////////////////////////////////////////////////////////////
				// Is the incoming 'photos_count' different than our existing 'photos_count'?
				// If so, let's process the media!
				//
				if($transformedData['base']['photos_count'] != $existingListing->photos_count) {
					throw new \Exception("Existing photos_count value ({$existingListing->photos_count}) NOT EQUAL to incoming photos_count value ({$transformedData['base']['photos_count']})");
				}

				$existingMediaCount = Ion3ListingMedia::where("ion_source", Ion3DataSource::CWS->getKey())
				                                      ->where("originating_system_name", $transformedData['base']['originating_system_name'])
				                                      ->where("resource_record_key", $transformedData['base']['listing_key'])
				                                      ->count();

				/////////////////////////////////////////////////////////////////////////////////////////
				// Do we have a different number of media records in our system for this listings than
				// what the source is telling us there should be? If so, let's process the media!
				//
				if($transformedData['base']['photos_count'] != $existingMediaCount) {
					throw new \Exception("Existing media record count ({$existingMediaCount}) NOT EQUAL expected count ({$transformedData['base']['photos_count']})");
				}
			}

		} catch (\Exception $e) {
			if(strstr($e->getMessage(), "No query results") !== false) {
				if(isset($transformedData['base']['photos_count']) && $transformedData['base']['photos_count'] > 0) {
					Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("Process Media: Adding new listing with incoming photos_count of {$transformedData['base']['photos_count']}");
					$transformedData['base']['process_media'] = 1;
				}
			} else {
				Log::channel(CWSClient::getLogChannel(CWSClient::$PropertyEntity))->info("Process Media: " . $e->getMessage());
				$transformedData['base']['process_media'] = 1;
			}
		}

		return $transformedData;
	}
}
