<?php

namespace App\Models;

use Filament\AvatarProviders\UiAvatarsProvider;
use Filament\Models\Contracts\FilamentUser;
use Filament\Models\Contracts\HasAvatar;
use Filament\Models\Contracts\HasName;
use Filament\Panel;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Storage;
use Laravel\Sanctum\HasApiTokens;
use Neutrino\Ion3Core\Database\Factories\UserFactory;
use Neutrino\Ion3Core\Events\Ion3RelationUpdated;
use Neutrino\Ion3Core\Events\Ion3Updated;
use Neutrino\Ion3Core\Traits\Ion3MetadataTrait;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;

class User extends Authenticatable implements FilamentUser, HasName, HasAvatar
{
    use HasApiTokens, HasFactory, Notifiable, Ion3MetadataTrait, SoftDeletes, LogsActivity;

    protected $guarded = [
        'id',
        'email_verified_at',
        'remember_token',
        'origdata',
        'deleted_at',
        'created_at',
        'updated_at',
    ];

    protected $hidden = [
        'password',
        'remember_token',
    ];

    protected $casts = [
        'active' => 'boolean',
        'is_team' => 'boolean',
        'show_online' => 'boolean',
        'email_verified_at' => 'datetime',
        'password' => 'hashed',
        'metadata' => 'array',
        'origdata' => 'array',
    ];

    protected $dispatchesEvents = [
        'updating' => Ion3Updated::class,
    ];

    protected static function newFactory()
    {
        return UserFactory::new();
    }

    public function agents(): HasMany
    {
        return $this->hasMany(Agent::class, 'user_id');
    }

    public function agentSources(): HasMany
    {
        return $this->hasMany(Ion3UserAgentSource::class, 'user_id');
    }

    public function companyLocations(): BelongsToMany
    {
        return $this->belongsToMany(CompanyLocation::class, 'company_location_user', 'user_id')
            ->withPivot('position')
            ->withTimestamps();
    }

    public function returnListings($additionalRelations = []) {
        $listings = Listing::whereHas('agents', function ($query) {
            $query->whereIn('agents.id', $this->agents->pluck('id'));
        });

        if (!empty($additionalRelations)) {
            $listings = $listings->with($additionalRelations);
        }

        return $listings;
    }

    public function getFirstCompanyLocationAvatarUrl()
    {
        $companyLocation = $this->companyLocations()->first();

        if ($companyLocation) {
            return $companyLocation->getFilamentAvatarUrl();
        }

        return CompanyLocation::getDefaultImage();
    }

    public function getFullName(): string
    {
        if (!empty($this->display_name)) {
            return $this->display_name;
        }

        // Familiar or First name
        $name = trim(($this->familiar_name) ?: $this->first_name);
        if (isset($this->metadata['middle_name'])) {
            $name .= ' ' . trim($this->metadata['middle_name']);
        }
        $name .= ' ' . $this->last_name;

        return trim($name);
    }

    public function getFilamentAvatarUrl(): ?string
    {
        // If photo_url is empty, return default
        if (empty($this->photo_url)) {
            return $this->getDefaultImage();
        }

        if (str_contains($this->photo_url, 'http')) {
            return $this->photo_url;
        }

        $disk = config('filament.default_filesystem_disk', 'public');
        if (!array_key_exists($disk, config('filesystems.disks'))) {
            $disk = 'public';
        }

        if (null !== config("filesystems.disks.{$disk}.url")) {
            return rtrim(config("filesystems.disks.{$disk}.url"), '/') . '/' . ltrim($this->photo_url, '/');
        } else {
            return Storage::disk($disk)->url($this->photo_url);
        }
    }

    static public function getDefaultImage() {
        return asset('images/default/user.jpg');
    }

    public function canAccessPanel(Panel $panel): bool
    {
        return true;
        //return str_ends_with($this->email, '@yourdomain.com') && $this->hasVerifiedEmail();
    }

    public function getFilamentName(): string
    {
        return $this->getFullName();
    }

    // Typesense Data Collection name
    public function searchableAs()
    {
        return 'users';
    }

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
            ->logExcept(['origdata', 'updated_at', 'metadata']) // really important
            ->logOnly(['*'])
            //->logOnly(['*', 'companyLocations.name', 'agentSources.source', 'agentSource.source_id'])
            ->logUnguarded()
            ->logOnlyDirty()
            ->dontSubmitEmptyLogs();
    }

    // TODO: Not being read at all
    static public function pivotUpdated($model, $relationName, $pivotIds, $pivotIdsAttributes)
    {
        echo 'pivotUpdated';
        echo get_class($model);
        echo $relationName;
        print_r($pivotIds);
        print_r($pivotIdsAttributes);
    }

}
